#========================================================================================
# package      DE_EPAGES::PayPalExpress::t::unit::WebService_BasketService_updateAddress
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for WebService::BasketService
#========================================================================================
package DE_EPAGES::PayPalExpress::t::unit::WebService_BasketService_updateAddress;
use base DE_EPAGES::Core::API::Test;
use strict;
use DE_EPAGES::Order::API::Object::Basket;
use DE_EPAGES::Permission::API::Object::User;
use DE_EPAGES::Test::API::TestTools qw ( TestExistsMethod );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::More;

sub test {
    my $self = shift;

    $self->test_updateBillingAddress();
    $self->test_updateShippingAddress();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_updateBillingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    $self->{'__delete_CallCounter'} = 0; # since the flag is stored global, we have to init the variable
    my $BasketService = $self->_createBasketService();
    my $Basket = $self->_buildBasket();
    my $WasImportSucessful = $BasketService->importBasket( $Basket );

    my $hBillingAddressData = {
        'FirstName'=> 'NewBillingName',
        'EMail' => 'e@mail.de',
    };

    $BasketService->updateBillingAddress( $hBillingAddressData );

    ok( $WasImportSucessful, "$SubTestName - tests if the basket import was sucessful" );
    my $aInsertChildParameters = $Basket->{'__insertChild_Parameters'};
    my $aExpectedInsertChildParameters = [
          'Address',
          { 'Alias' => 'BillingAddress',
            'FirstName' => 'NewBillingName',
            'EMail' => 'e@mail.de',
          }
        ];
    is_deeply(
        $aInsertChildParameters,
        $aExpectedInsertChildParameters,
        "$SubTestName - tests if InsertChild has been called with the expected params"
    );
    isa_ok(
        $Basket->get( 'BillingAddress' ),
        'DE_EPAGES::Address::API::Object::Address',
        "$SubTestName - tests if the BillingAddress is set correctly in the Basket"
    );

    is( $Basket->{ '__existsChild_Parameters' }->[0], 'BillingAddress', "$SubTestName - tests if the existsChild method has been called with the expected params" );
    is( $self->{ '__delete_CallCounter' }, 1, "$SubTestName - tests if delete was called" );
}
#----------------------------------------------------------------------------------------
sub test_updateShippingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    $self->{'__delete_CallCounter'} = 0; # since the flag is stored global, we have to init the variable
    my $BasketService = $self->_createBasketService();
    my $Basket = $self->_buildBasket();
    my $WasImportSucessful = $BasketService->importBasket( $Basket );

    my $hShippingAddressData = {
        'FirstName'=> 'NewFirstName',
        'LastName' => 'NewLastName',
    };

    $BasketService->updateShippingAddress( $hShippingAddressData );

    ok( $WasImportSucessful, "$SubTestName - tests if the basket import was sucessful" );
    my $aInsertChildParameters = $Basket->{'__insertChild_Parameters'};
    my $aExpectedInsertChildParameters = [
          'Address',
          { 'Alias' => 'ShippingAddress',
            'FirstName' => 'NewFirstName',
            'LastName' => 'NewLastName',
          }
    ];
    is_deeply(
        $aInsertChildParameters,
        $aExpectedInsertChildParameters,
        "$SubTestName - tests if InsertChild has been called with the expected params"
    );
    my $ShippingAddress = $Basket->get( 'ShippingAddress' );
    isa_ok(
        $ShippingAddress,
        'DE_EPAGES::Address::API::Object::Address',
        "$SubTestName - tests if the ShippingAddress is set correctly in the Basket"
    );
    is( $Basket->{'__existsChild_Parameters'}->[0], 'ShippingAddress', "$SubTestName - tests if the existsChild method has been called with the expected params" );
    is( $self->{'__delete_CallCounter'}, 1, "$SubTestName - tests if delete was called" );
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createBasketService {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::BasketService',
                   $aParameterList
              );

    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_UpdateTaxModel', undef,
        [{'object' => 'DE_EPAGES::Order::API::Object::LineItemContainer'}]
    );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildBasket {
    my $self = shift;

    my $User = $self->_createUser( $self->_createAddressFolder() );
    my $hAttributes = {
        'ID' => 123456,
        'Alias' => 'MyTestAddress',
        'instanceOf'=>'Basket',
        'LineItemContainer' => $self->_createLineItemContainer(),
        'User' => $User,
        'BillingAddress' => $self->_buildBillingAddress(),
        'ShippingAddress' => $self->_buildShippingAddress(),
    };
    my $MockedBasket = $self->_createBasket( $hAttributes );

    return $MockedBasket;
}
#----------------------------------------------------------------------------------------
sub _createUser {
    my $self = shift;
    my ( $AddressFolderReturn ) = @_;

    my $hAttributes = {
        'Alias' => 'UserName',
        'ID' => 987654,
        'ShippingAddress' => $self->_buildShippingAddress(),
        'BillingAddress' => $self->_buildBillingAddress(),
    };
    my $User = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Permission::API::Object::User',
        $hAttributes
    );

    TestExistsMethod( 'DE_EPAGES::Permission::API::Object::User','folder' );
    $User->mock( 'folder',sub{
        shift;
        $User->{'__folder_Parameters'} = \@_;
        return $AddressFolderReturn;
    });

    return $User;
}
#----------------------------------------------------------------------------------------
sub _createAddressFolder {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::Address::API::Object::Address',
                   $aParameterList
    );
    $Mockify->addMockWithReturnValue( 'children', $self->_buildChildrenAddresses() );
    $Mockify->addMockWithReturnValueAndParameterCheck( 'nextAlias', 'Address_1', [{'string' => 'Address_%d'}] );
    $Mockify->addMockWithReturnValueAndParameterCheck( 'insertChild', undef, [{ 'string' => 'Address' }, 'hashref'] );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'Payment' => $self->_createPayPalExpressLineItemPayment(),
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer',
        $hAttributes
    );

    return $LineItemContainer;
}
#----------------------------------------------------------------------------------------
sub _createPayPalExpressLineItemPayment {
    my $self = shift;

    my $hAttributes = {};
    my $_createPayPalExpressLineItemPayment = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::PayPalExpress::API::Object::LineItemPaymentPayPalExpress',
        $hAttributes
    );

    return $_createPayPalExpressLineItemPayment;
}
#----------------------------------------------------------------------------------------
sub _buildChildrenAddresses {
    my $self = shift;

    my $hAttributesChild1 = {
        'ID' => 1234,
        'FirstName' => 'ChildFirstName_1',
        'LastName' => 'ChildLastName_1',
        'Parent' => 'ThisIsAParent',
    };
    my $Child1 = $self->_createAddressObject( $hAttributesChild1 );
    my $hAttributesChild2 = {
        'ID' => 1234,
        'FirstName' => 'ChildFirstName_2',
        'LastName' => 'ChildLastName_2',
        'Parent' => 'ThisIsAParent',
    };
    my $Child2 = $self->_createAddressObject( $hAttributesChild2 );

    return [$Child1, $Child2];
}


#----------------------------------------------------------------------------------------
sub _buildShippingAddress {
    my $self = shift;

    my $hAttributes = {
        'ID' => 1234,
        'FirstName' => 'DefaultFirstName',
        'LastName' => 'DefaultLastName',
        'Parent' => 'ThisIsAParent',
    };

    return $self->_createAddressObject($hAttributes);
}
#----------------------------------------------------------------------------------------
sub _buildBillingAddress {
    my $self = shift;

    my $hAttributes = {
        'ID' => 1234,
        'FirstName' => 'DefaultFirstName',
        'EMail' => 'DefaultLastName',
        'Parent' => 'ThisIsAParent',
    };

    return $self->_createAddressObject($hAttributes);
}
#----------------------------------------------------------------------------------------
sub _createAddressObject {
    my $self = shift;
    my ($hAttributes) = @_;

    my $AddressObject = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Address::API::Object::Address',
        $hAttributes
    );

    TestExistsMethod('DE_EPAGES::Object::API::Object::Object','delete');
    $AddressObject->mock('delete', sub {
        shift;
        # Since we override the Object later we have to store the flag global
        $self->{'__delete_CallCounter'} += 1;
        return;
    });

    return $AddressObject;
}
#----------------------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;
    my ($hAttributes) = @_;

    my $Basket = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket',
        $hAttributes
    );
    TestExistsMethod('DE_EPAGES::Order::API::Object::Basket','existsChild');
    $Basket->mock('existsChild',sub{
        shift;
        $Basket->{'__existsChild_Parameters'} = \@_;
        return 1;
    });
    TestExistsMethod('DE_EPAGES::Order::API::Object::Basket','insertChild');
    $Basket->mock('insertChild',sub{
        shift;
        $Basket->{'__insertChild_Parameters'} = \@_;
        return $self->_createAddressObject();
    });

    return $Basket;
}

__PACKAGE__->runScript();