#========================================================================================
# package      DE_EPAGES::PayPalExpress::t::unit::Webservice_AddressService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for Webservice::AddressService
#========================================================================================
package DE_EPAGES::PayPalExpress::t::unit::Webservice_AddressService;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::PayPalExpress::API::Webservice::AddressService;
use Test::More;
use DE_EPAGES::Test::Mock::Mockify;
use strict;

sub test {
    my $self = shift;

    $self->test_updateAddresses();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_updateAddresses {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_createBasketService();
    my $AddressService = $self->_createAddressService( $BasketService );
    my $ResponseObject = $self->_createResponseObject();

    $AddressService->updateAddresses( $ResponseObject );

    is( $BasketService->{'__refresh_WasCalled'}, 1, "$SubTestName - explain what is tested" );

    return;
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createAddressService {
    my $self = shift;
    my ($BasketService) = @_;

    my $aParameterList = [ $BasketService ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::AddressService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_ExistsCountryByCode2', 1, [{'string'=>'USA'}] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_GetPKeyCountryByCode2', 'US', [{'string'=>'USA'}] );
    $Mockify->addMockWithReturnValue( '_createFullNameObject', $self->_createFullNameObject() );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createBasketService {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::BasketService',
                   $aParameterList,
              );
    $Mockify->addMock( 'refresh', sub {
        my $MockedSelf = shift;
        $MockedSelf->{'__refresh_WasCalled'} += 1;
        return;
    } );

    my $hExpectedBillingAddressData = {
        'CountryID'=> 'US'
    };
    $Mockify->addMockWithReturnValueAndParameterCheck( 'updateBillingAddress', undef, [ {'hashref' => $hExpectedBillingAddressData} ] );
    my $hExpectedShippingAddressData = {
        'CountryID'=> 'US',
        'FirstName' => 'aFirstName',
        'LastName' => 'aLastName',
    };
    $Mockify->addMockWithReturnValueAndParameterCheck( 'updateShippingAddress', undef, [ {'hashref' => $hExpectedShippingAddressData} ] );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createResponseObject {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::Response',
                   $aParameterList
              );

    my $ReturnValue = { 'CountryID'=> 'USA' };
    $Mockify->addMockWithReturnValue( 'getBillingAddressData', $ReturnValue );
    my $hShippingAddressData = {
        'CountryID'=> 'USA',
        'LastName' => 'Mueller',
    };
    $Mockify->addMockWithReturnValue( 'getShippingAddressData', $hShippingAddressData );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createFullNameObject {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::FullName',
                   $aParameterList
              );
    my $WasSeperationFine = 1;
    $Mockify->addMockWithReturnValueAndParameterCheck( 'separateNames', $WasSeperationFine, ['string'] );
    $Mockify->addMockWithReturnValue( 'getFirstName', 'aFirstName' );
    $Mockify->addMockWithReturnValue( 'getLastName', 'aLastName' );

    return $Mockify->getMockObject();
}
__PACKAGE__->runScript();