#========================================================================================
# package      DE_EPAGES::PayPalExpress::t::unit::Webservice_FullName
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for Webservice_FullName
#========================================================================================
package DE_EPAGES::PayPalExpress::t::unit::Webservice_FullName;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use strict;
use DE_EPAGES::PayPalExpress::API::Webservice::FullName;
use DE_EPAGES::PayPalExpress::API::Constants qw ( $MAXIMUM_NUMBER_ALLOWED_CHARACTERS );
sub test {
    my $self = shift;

    $self->test_organizeNames_OneNameElement_EmptyName();
    $self->test_organizeNames_OneNameElement_NormalNameSize();
    $self->test_organizeNames_OneNameElement_OverSize();
    $self->test_organizeNames_TwoElements();
    $self->test_organizeNames_TwoElementsWithTooLongName();
    $self->test_organizeNames_extractNamesForMoreThanTwoElementsAndTooLongNames();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub init {
    # because the test needs to be independent of the actual setting in the constant.pm this is set to 15 Characters
    $MAXIMUM_NUMBER_ALLOWED_CHARACTERS = 15; ## no critic (ProhibitMagicNumbers)
}
#----------------------------------------------------------------------------------------
sub test_organizeNames_OneNameElement_EmptyName {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $FullName = DE_EPAGES::PayPalExpress::API::Webservice::FullName->new();
    my $LastName = '';

    my $WasSeparationSuccessful = $FullName->separateNames( $LastName );

    is( $WasSeparationSuccessful, 0, "$SubTestName - tests if an empty name is not valid" );

}

#----------------------------------------------------------------------------------------
sub test_organizeNames_OneNameElement_NormalNameSize {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $FullName = DE_EPAGES::PayPalExpress::API::Webservice::FullName->new();
    my $LastName = 'ASingleFullName';

    my $WasSeparationSuccessful = $FullName->separateNames( $LastName );

    ok( $WasSeparationSuccessful, "$SubTestName - tests if validation works" );
    is( $FullName->getFirstName(), 'ASingle', "$SubTestName - tests if ASingleFullName is split correctly for firstname" );
    is( $FullName->getLastName(), 'FullName', "$SubTestName - tests if ASingleFullName is split correctly for lastname" );

}
#----------------------------------------------------------------------------------------
sub test_organizeNames_OneNameElement_OverSize {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $FullName = DE_EPAGES::PayPalExpress::API::Webservice::FullName->new();
    my $LastName = 'ThisIsALongFirstNameWithAllotLetters';

    my $WasSeparationSuccessful = $FullName->separateNames( $LastName );

    ok( $WasSeparationSuccessful, "$SubTestName - tests if validation works" );
    is( $FullName->getFirstName(), 'ThisIsALongFirs', "$SubTestName - tests if the firstname contains the first expected characters" );
    is( $FullName->getLastName(), 'meWithAllotLett', "$SubTestName - tests if the lastname contains the expected characters" );

}
#----------------------------------------------------------------------------------------
sub test_organizeNames_TwoElements {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $FullName = DE_EPAGES::PayPalExpress::API::Webservice::FullName->new();
    my $LastName = 'Joe Miller';

    my $WasSeparationSuccessful = $FullName->separateNames( $LastName );

    ok( $WasSeparationSuccessful, "$SubTestName - tests if validation works" );
    is( $FullName->getFirstName(), 'Joe', "$SubTestName - tests if FirstName is split correctly" );
    is( $FullName->getLastName(), 'Miller', "$SubTestName - tests if LastName is split correctly" );
}
#----------------------------------------------------------------------------------------
sub test_organizeNames_TwoElementsWithTooLongName {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $FullName = DE_EPAGES::PayPalExpress::API::Webservice::FullName->new();
    my $LastName = 'TooLongFirstName TooooLongLastName';

    my $WasSeparationSuccessful = $FullName->separateNames( $LastName );

    ok( $WasSeparationSuccessful, "$SubTestName - tests if validation works" );
    is( $FullName->getFirstName(), 'TooLongFirstNam', "$SubTestName - tests if FirstName is split and croped correctly" );
    is( $FullName->getLastName(), 'TooooLongLastNa', "$SubTestName - tests if LastName is split and croped correctly" );
}
#----------------------------------------------------------------------------------------
sub test_organizeNames_extractNamesForMoreThanTwoElementsAndTooLongNames {
    my $self = shift;

    my $SubTestName = (caller(0))[3];
    my $FullName = DE_EPAGES::PayPalExpress::API::Webservice::FullName->new();
    my $LastName = 'Dr.Lord Joe John Bob van Miller-Mayer House';

    my $WasSeparationSuccessful = $FullName->separateNames( $LastName );

    ok( $WasSeparationSuccessful, "$SubTestName - tests if validation works" );
    is( $FullName->getFirstName(), 'Dr.Lord Joe Joh', "$SubTestName - tests the firstname seperation for multiple names" );
    is( $FullName->getLastName(), 'Bob van Miller-', "$SubTestName - tests the lastname seperation for multiple names" );
}

__PACKAGE__->runScript();