#========================================================================================
# package      DE_EPAGES::PayPalExpress::t::unit::Webservice_GetExpressCheckoutDetails
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for Webservice::GetExpressCheckoutDetails
#========================================================================================
package DE_EPAGES::PayPalExpress::t::unit::Webservice_GetExpressCheckoutDetails;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::PayPalExpress::API::Webservice::GetExpressCheckoutDetails;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::More;
use Test::Exception;
use strict;

sub test {
    my $self = shift;

    $self->test_doRequestWithValidParams();
    $self->test_doRequestHandleError();
    $self->test_doRequestHandleGeneralError();
    $self->test_doRequestForShortcutButtonCheckoutSuccessfulUpdateAddresses();
    $self->test_doRequestForShortcutButtonCheckoutHandleError();
    $self->test_doRequestForShortcutButtonCheckoutHandleGeneralError();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_doRequestWithValidParams {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $BasketService = $self->_createBasketService();

    my $GetExpressCheckoutDetails = $self->_createGetExpressCheckoutDetails();

    my $Result = $GetExpressCheckoutDetails->doRequestForClassicOrMultistepCheckout( $BasketService );
    ok( $Result, "$SubTestName - doRequest successful" );
}
#----------------------------------------------------------------------------------------
sub test_doRequestHandleError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $BasketService = $self->_createBasketService();


    my $GetExpressCheckoutDetails = $self->_createGetExpressCheckoutDetailsWithError( 'PayPalExpressFailed' );
    is( $GetExpressCheckoutDetails->doRequestForClassicOrMultistepCheckout( $BasketService ), 0, "$SubTestName - tests if error is correctly handled" );

    is( $GetExpressCheckoutDetails->getErrorCode(), 'PayPalExpressFailed', "$SubTestName - check if ErrorCode is correctly stored" );
    is( $GetExpressCheckoutDetails->getErrorMessage(), 'PayPalExpressFailed: ErrorMessage=_CheckoutRequestDetails not successful', "$SubTestName - check if the ErrorMessage is correctly set" );
}
#----------------------------------------------------------------------------------------
sub test_doRequestHandleGeneralError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $BasketService = $self->_createBasketService();


    my $GetExpressCheckoutDetails = $self->_createGetExpressCheckoutDetailsWithError( 'PayPalExpressError' );

    throws_ok( sub { $GetExpressCheckoutDetails->doRequestForClassicOrMultistepCheckout( $BasketService ); },
                qr/PayPalExpressError: PayPalExpressError: ErrorMessage=_CheckoutRequestDetails not successful/, ## no critic (ProhibitComplexRegexes)
                  "$SubTestName - test if general error is correctly handled"
    );
}
#----------------------------------------------------------------------------------------
sub test_doRequestForShortcutButtonCheckoutSuccessfulUpdateAddresses {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_createBasketService();

    my $GetExpressCheckoutDetails = $self->_createGetExpressCheckoutDetails();

    ok( $GetExpressCheckoutDetails->doRequestForShortcutButtonCheckout( $BasketService ), "$SubTestName - doRequestForShortcutButtonCheckout runs successful" );

}
#----------------------------------------------------------------------------------------
sub test_doRequestForShortcutButtonCheckoutHandleError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $BasketService = $self->_createBasketService();


    my $GetExpressCheckoutDetails = $self->_createGetExpressCheckoutDetailsWithError('PayPalExpressFailed');
    is( $GetExpressCheckoutDetails->doRequestForShortcutButtonCheckout( $BasketService ), 0, "$SubTestName - tests if error is correctly handled" );

    is( $GetExpressCheckoutDetails->getErrorCode(), 'PayPalExpressFailed',"$SubTestName - check if ErrorCode is correctly stored" );
    is( $GetExpressCheckoutDetails->getErrorMessage(), 'PayPalExpressFailed: ErrorMessage=_CheckoutRequestDetails not successful',"$SubTestName - check if the ErrorMessage is correctly set" );
}
#----------------------------------------------------------------------------------------
sub test_doRequestForShortcutButtonCheckoutHandleGeneralError{
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $BasketService = $self->_createBasketService();


    my $GetExpressCheckoutDetails = $self->_createGetExpressCheckoutDetailsWithError( 'PayPalExpressError' );

    throws_ok( sub { $GetExpressCheckoutDetails->doRequestForShortcutButtonCheckout( $BasketService ); },
                qr/PayPalExpressError: PayPalExpressError: ErrorMessage=_CheckoutRequestDetails not successful/, ## no critic (ProhibitComplexRegexes)
                  "$SubTestName - test if general error is correctly handled"
    );
}
#----------------------------------------------------------------------------------------
#HELPERS
#----------------------------------------------------------------------------------------
sub _createBasketService {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::BasketService',
                   $aParameterList
    );
    my $BasketID = 658; ## no critic (ProhibitMagicNumbers)
    $Mockify->addMockWithReturnValue( 'getBasketID', $BasketID );
    $Mockify->addMockWithReturnValue( 'getLineItemPayment', $self->_createLineItemPaymentPayPalExpress() );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createLineItemPaymentPayPalExpress {
    my $self = shift;

    my $hAttributes = {};
    my $LineItemPaymentPayPalExpress = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::PayPalExpress::API::Object::LineItemPaymentPayPalExpress',
        $hAttributes
    );

    return $LineItemPaymentPayPalExpress;
}
#----------------------------------------------------------------------------------------
sub _createGetExpressCheckoutDetailsWithError {
    my $self = shift;
    my ( $ErrorType ) = @_;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::GetExpressCheckoutDetails',
                   $aParameterList
    );
    my $hVars = {'ErrorMessage' => '_CheckoutRequestDetails not successful' };

    $Mockify->addMock( '_CheckoutRequestDetails', sub { Error($ErrorType, $hVars); } );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createGetExpressCheckoutDetails {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::GetExpressCheckoutDetails',
                   $aParameterList
    );

    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_createAddressService',
        $self->_createAddressService(),
        [ {'object' => 'DE_EPAGES::PayPalExpress::API::Webservice::BasketService'}, ]
    );
    my $BasketObjectID = 658; ## no critic (ProhibitMagicNumbers)
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_CheckoutRequestDetails',
        {'PayPalResponse'=>'aResponse'},
        [
            {'object'=> 'DE_EPAGES::PayPalExpress::API::Object::LineItemPaymentPayPalExpress'},
            {'hashref' => {'ObjectID' => $BasketObjectID}}
        ]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_createResponseObject',
        $self->_createResponseObject(),
        [{'hashref'=> {'PayPalResponse'=>'aResponse'}}]
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createResponseObject {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::Response',
                   $aParameterList
              );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createAddressService {
    my $self = shift;
    my ( $WasAddressSuccessfulUpdated ) = @_;
    my $aParameterList = [ $self->_createBasketService() ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::AddressService',
                   $aParameterList
    );

    $Mockify->addMockWithReturnValueAndParameterCheck(
        'updateAddresses',
        undef,
        [{'object'=>'DE_EPAGES::PayPalExpress::API::Webservice::Response'}]
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();